@@ -24,16 +24,21 @@
  * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include <stdio.h>
+#include <paths.h>
 #include <Python.h>
+#include <math.h>
+#include <bytesobject.h>
+#include <liberasurecode/erasurecode.h>
+
+#if ( LIBERASURECODE_VERSION < _VERSION(1,1,0) )
+#include <liberasurecode/erasurecode_helpers.h>
+#endif
 
 /* Compat layer for python <= 2.6 */
 #include "capsulethunk.h"
 
-#include <erasurecode.h>
-#include <erasurecode_helpers.h>
-#include <math.h>
 #include <pyeclib_c.h>
-#include <bytesobject.h>
 
 /* Python 3 compatibility macros */
 #if PY_MAJOR_VERSION >= 3
@@ -46,7 +51,7 @@
               PyModuleDef_HEAD_INIT, name, doc, -1, methods, }; \
           ob = PyModule_Create(&moduledef);
   #define PY_BUILDVALUE_OBJ_LEN(obj, objlen) \
-          Py_BuildValue("y#", obj, objlen)
+          Py_BuildValue("y#", obj, (Py_ssize_t)objlen)
   #define PyInt_FromLong PyLong_FromLong
   #define PyString_FromString PyUnicode_FromString
   #define ENCODE_ARGS "Oy#"
@@ -58,7 +63,7 @@
   #define MOD_DEF(ob, name, doc, methods) \
           ob = Py_InitModule3(name, methods, doc);
   #define PY_BUILDVALUE_OBJ_LEN(obj, objlen) \
-          Py_BuildValue("s#", obj, objlen)
+          Py_BuildValue("s#", obj, (Py_ssize_t)objlen)
   #define ENCODE_ARGS "Os#"
   #define GET_METADATA_ARGS "Os#i"
 #endif
@@ -87,6 +92,51 @@ static PyObject *import_class(const char *module, const char *cls)
     return (PyObject *) PyObject_GetAttrString(s, cls);
 }
 
+/**
+ * Allocate a buffer of a specific size and set its' contents
+ * to the specified value.
+ *
+ * @param size integer size in bytes of buffer to allocate
+ * @param value
+ * @return pointer to start of allocated buffer or NULL on error
+ */
+void * alloc_and_set_buffer(int size, int value) {
+    void * buf = NULL;  /* buffer to allocate and return */
+  
+    /* Allocate and zero the buffer, or set the appropriate error */
+    buf = malloc((size_t) size);
+    if (buf) {
+        buf = memset(buf, value, (size_t) size);
+    }
+    return buf;
+}
+
+/**
+ * Allocate a zero-ed buffer of a specific size.
+ *
+ * @param size integer size in bytes of buffer to allocate
+ * @return pointer to start of allocated buffer or NULL on error
+ */
+void * alloc_zeroed_buffer(int size)
+{
+    return alloc_and_set_buffer(size, 0);
+}
+
+/**
+ * Deallocate memory buffer if it's not NULL.  This methods returns NULL so 
+ * that you can free and reset a buffer using a single line as follows:
+ *
+ * my_ptr = check_and_free_buffer(my_ptr);
+ *
+ * @return NULL
+ */
+void * check_and_free_buffer(void * buf)
+{
+    if (buf)
+        free(buf);
+    return NULL;
+}
+
 void
 pyeclib_c_seterr(int ret, const char * prefix)
 {
@@ -140,13 +190,39 @@ pyeclib_c_seterr(int ret, const char * prefix)
     PyErr_SetString(eo, err);
 }
 
+static int stderr_fd;
+static fpos_t stderr_fpos;
+#ifndef _PATH_DEVNULL
+#define _PATH_DEVNULL "/dev/null"
+#endif
+
+static void redirect_stderr(void)
+{
+    fflush(stderr);
+    fgetpos(stderr, &stderr_fpos);
+    stderr_fd = dup(fileno(stderr));
+    freopen(_PATH_DEVNULL, "w", stderr);
+}
+
+static void restore_stderr(void)
+{
+    fflush(stderr);
+    dup2(stderr_fd, fileno(stderr));
+    close(stderr_fd);
+    clearerr(stderr);
+    fsetpos(stderr, &stderr_fpos);        /* for C9X */
+}
+
 /**
  * Constructor method for creating a new pyeclib object using the given parameters.
  *
  * @param k integer number of data elements
  * @param m integer number of checksum elements
- * @param w integer word size in bytes
+ * @param hd hamming distance
  * @param backend_id erasure coding backend
+ * @param use_inline_chksum type of inline fragment header checksum
+ * @param use_algsig_chksum use algorithmic signature for fragment header checksum
+ * @param validate only validate backend and params, close handle immediately
  * @return pointer to PyObject or NULL on error
  */
 static PyObject *
@@ -154,12 +230,14 @@ pyeclib_c_init(PyObject *self, PyObject *args)
 {
   pyeclib_t *pyeclib_handle = NULL;
   PyObject *pyeclib_obj_handle = NULL;
-  int k, m, hd=0;
+  int k, m, hd = 0, validate = 0;
   int use_inline_chksum = 0, use_algsig_chksum = 0;
   const ec_backend_id_t backend_id;
 
   /* Obtain and validate the method parameters */
-  if (!PyArg_ParseTuple(args, "iii|iii", &k, &m, &backend_id, &hd, &use_inline_chksum, &use_algsig_chksum)) {
+  if (!PyArg_ParseTuple(args, "iii|iiiii",
+                        &k, &m, &backend_id, &hd, &use_inline_chksum,
+                        &use_algsig_chksum, &validate)) {
     pyeclib_c_seterr(-EINVALIDPARAMS, "pyeclib_c_init ERROR: ");
     return NULL;
   }
@@ -168,7 +246,7 @@ pyeclib_c_init(PyObject *self, PyObject *args)
   pyeclib_handle = (pyeclib_t *) alloc_zeroed_buffer(sizeof(pyeclib_t));
   if (NULL == pyeclib_handle) {
     pyeclib_c_seterr(-ENOMEM, "pyeclib_c_init ERROR: ");
-    goto error;
+    goto cleanup;
   }
 
   pyeclib_handle->ec_args.k = k;
@@ -176,10 +254,15 @@ pyeclib_c_init(PyObject *self, PyObject *args)
   pyeclib_handle->ec_args.hd = hd;
   pyeclib_handle->ec_args.ct = use_inline_chksum ? CHKSUM_CRC32 : CHKSUM_NONE;
 
+  if (validate)
+    redirect_stderr();
+
   pyeclib_handle->ec_desc = liberasurecode_instance_create(backend_id, &(pyeclib_handle->ec_args));  
   if (pyeclib_handle->ec_desc <= 0) {
-    pyeclib_c_seterr(-EINVALIDPARAMS, "pyeclib_c_init ERROR: ");
-    goto error; 
+    /* liberasurecode returns status in ec_desc as one of the error codes
+     * (LIBERASURECODE_ERROR_CODES) defined in erasurecode.h */
+    pyeclib_c_seterr(pyeclib_handle->ec_desc, "pyeclib_c_init ERROR: ");
+    goto cleanup;
   }
 
   /* Prepare the python object to return */
@@ -187,27 +270,28 @@ pyeclib_c_init(PyObject *self, PyObject *args)
   pyeclib_obj_handle = PyCapsule_New(pyeclib_handle, PYECC_HANDLE_NAME,
                                      pyeclib_c_destructor);
 #else
-  pyeclib_obj_handle = PyCObject_FromVoidPtrAndDesc(pyeclib_handle,
-                         (void *) PYECC_HANDLE_NAME,
-             pyeclib_c_destructor);
+  pyeclib_obj_handle = PyCObject_FromVoidPtrAndDesc(
+                                     pyeclib_handle, (void *) PYECC_HANDLE_NAME,
+                                     pyeclib_c_destructor);
 #endif /* Py_CAPSULE_H */
 
   /* Clean up the allocated memory on error, or update the ref count */
   if (pyeclib_obj_handle == NULL) {
     pyeclib_c_seterr(-EINVALIDPARAMS, "pyeclib_c_init ERROR: ");
-    goto error;
+    goto cleanup;
   } else {
     Py_INCREF(pyeclib_obj_handle);
   }
   
-  goto exit;
+exit:
+  if (validate)
+    restore_stderr();
+  return pyeclib_obj_handle;
 
-error:
+cleanup:
   check_and_free_buffer(pyeclib_handle);
   pyeclib_obj_handle = NULL;
-
-exit:  
-  return pyeclib_obj_handle;
+  goto exit;
 }
 
 
