@@ -27,11 +27,12 @@ import sys
 import tempfile
 import unittest
 from pyeclib.ec_iface import ECBackendNotSupported
+from pyeclib.ec_iface import ECDriver
 from pyeclib.ec_iface import ECDriverError
 from pyeclib.ec_iface import ECInsufficientFragments
-
-from pyeclib.ec_iface import ECDriver, PyECLib_EC_Types
-from .test_pyeclib_c import _available_backends
+from pyeclib.ec_iface import PyECLib_EC_Types
+from pyeclib.ec_iface import ALL_EC_TYPES
+from pyeclib.ec_iface import VALID_EC_TYPES
 
 if sys.version < '3':
     def b2i(b):
@@ -107,11 +108,27 @@ class TestPyECLibDriver(unittest.TestCase):
     def tearDown(self):
         pass
 
+    def test_valid_ec_types(self):
+        # Build list of available types and compare to VALID_EC_TYPES
+        available_ec_types = []
+        for _type in ALL_EC_TYPES:
+            try:
+                if _type is 'shss':
+                    _m = 4
+                else:
+                    _m = 5
+                driver = ECDriver(k=10, m=_m, ec_type=_type, validate=True)
+                available_ec_types.append(_type)
+            except Exception as e:
+                # ignore any errors, assume backend not available
+                continue
+        self.assertEqual(available_ec_types, VALID_EC_TYPES)
+
     def test_valid_algo(self):
         print("")
-        for _type in PyECLib_EC_Types.names():
+        for _type in ALL_EC_TYPES:
             # Check if this algo works
-            if _type not in _available_backends:
+            if _type not in VALID_EC_TYPES:
                 print("Skipping test for %s backend" % _type)
                 continue
             try:
@@ -128,7 +145,7 @@ class TestPyECLibDriver(unittest.TestCase):
     def get_pyeclib_testspec(self, csum="none"):
         pyeclib_drivers = []
         _type1 = 'jerasure_rs_vand'
-        if _type1 in _available_backends:
+        if _type1 in VALID_EC_TYPES:
             pyeclib_drivers.append(ECDriver(k=12, m=2, ec_type=_type1,
                                    chksum_type=csum))
             pyeclib_drivers.append(ECDriver(k=11, m=2, ec_type=_type1,
@@ -138,7 +155,7 @@ class TestPyECLibDriver(unittest.TestCase):
             pyeclib_drivers.append(ECDriver(k=8, m=4, ec_type=_type1,
                                    chksum_type=csum))
         _type2 = 'liberasurecode_rs_vand'
-        if _type2 in _available_backends:
+        if _type2 in VALID_EC_TYPES:
             pyeclib_drivers.append(ECDriver(k=12, m=2, ec_type=_type2,
                                    chksum_type=csum))
             pyeclib_drivers.append(ECDriver(k=11, m=2, ec_type=_type2,
@@ -147,11 +164,25 @@ class TestPyECLibDriver(unittest.TestCase):
                                    chksum_type=csum))
             pyeclib_drivers.append(ECDriver(k=8, m=4, ec_type=_type2,
                                    chksum_type=csum))
-        _type3 = 'flat_xor_hd'
-        if _type3 in _available_backends:
-            pyeclib_drivers.append(ECDriver(k=12, m=6, ec_type=_type3,
+        _type3_1 = 'flat_xor_hd'
+        if _type3_1 in VALID_EC_TYPES:
+            pyeclib_drivers.append(ECDriver(k=12, m=6, ec_type=_type3_1,
                                    chksum_type=csum))
-            pyeclib_drivers.append(ECDriver(k=10, m=5, ec_type=_type3,
+            pyeclib_drivers.append(ECDriver(k=10, m=5, ec_type=_type3_1,
+                                   chksum_type=csum))
+        _type3_2 = 'flat_xor_hd_4'
+        if _type3_2 in VALID_EC_TYPES:
+            pyeclib_drivers.append(ECDriver(k=12, m=6, ec_type=_type3_2,
+                                   chksum_type=csum))
+            pyeclib_drivers.append(ECDriver(k=10, m=5, ec_type=_type3_2,
+                                   chksum_type=csum))
+        _type4 = 'shss'
+        if _type4 in VALID_EC_TYPES:
+            pyeclib_drivers.append(ECDriver(k=10, m=4, ec_type=_type4,
+                                   chksum_type=csum))
+            pyeclib_drivers.append(ECDriver(k=20, m=4, ec_type=_type4,
+                                   chksum_type=csum))
+            pyeclib_drivers.append(ECDriver(k=11, m=7, ec_type=_type4,
                                    chksum_type=csum))
         return pyeclib_drivers
 
@@ -230,7 +261,7 @@ class TestPyECLibDriver(unittest.TestCase):
 
     def check_metadata_formatted(self, k, m, ec_type, chksum_type):
 
-        if ec_type not in _available_backends:
+        if ec_type not in VALID_EC_TYPES:
             return
 
         filesize = 1024 * 1024 * 3
@@ -351,6 +382,8 @@ class TestPyECLibDriver(unittest.TestCase):
 
     def test_get_segment_byterange_info(self):
         pyeclib_drivers = self.get_pyeclib_testspec()
+        if not pyeclib_drivers:
+            return
 
         file_size = 1024 * 1024
         segment_size = 3 * 1024
@@ -512,7 +545,7 @@ class TestPyECLibDriver(unittest.TestCase):
         if ec_type[:11] == "flat_xor_hd":
             return ECDriver(k=k, m=m, ec_type="flat_xor_hd",
                             chksum_type=chksum_type)
-        elif ec_type in _available_backends:
+        elif ec_type in VALID_EC_TYPES:
             return ECDriver(k=k, m=m, ec_type=ec_type, chksum_type=chksum_type)
         else:
             return None
@@ -523,18 +556,22 @@ class TestPyECLibDriver(unittest.TestCase):
         tmp_file.seek(0)
         whole_file_bytes = tmp_file.read()
         for ec_type in ['flat_xor_hd_3', 'liberasurecode_rs_vand']:
-            pyeclib_driver = self.get_available_backend(
-                k=10, m=5, ec_type=ec_type)
-            fragments = pyeclib_driver.encode(whole_file_bytes)
-            self.assertRaises(ECInsufficientFragments,
-                              pyeclib_driver.reconstruct,
-                              [fragments[0]], [1, 2, 3, 4, 5, 6])
+            if ec_type in VALID_EC_TYPES:
+                pyeclib_driver = self.get_available_backend(
+                    k=10, m=5, ec_type=ec_type)
+                fragments = pyeclib_driver.encode(whole_file_bytes)
+                self.assertRaises(ECInsufficientFragments,
+                                  pyeclib_driver.reconstruct,
+                                  [fragments[0]], [1, 2, 3, 4, 5, 6])
 
     def test_min_parity_fragments_needed(self):
         pyeclib_drivers = []
-        pyeclib_drivers.append(ECDriver(k=12, m=2, ec_type="liberasurecode_rs_vand"))
-        self.assertTrue(
-            pyeclib_drivers[0].min_parity_fragments_needed() == 1)
+        for ec_type in ['flat_xor_hd_3', 'liberasurecode_rs_vand']:
+            if ec_type in VALID_EC_TYPES:
+                pyeclib_drivers.append(ECDriver(k=10, m=5, ec_type=ec_type))
+                self.assertTrue(
+                    pyeclib_drivers[0].min_parity_fragments_needed() == 1)
+
 
 if __name__ == '__main__':
     unittest.main()
